==> ozone-1.3.0-src.tar.gz: ozone-1.3.0-src/hadoop-hdds/erasurecode/src/main/java/org/apache/ozone/erasurecode/rawcoder/AbstractNativeRawEncoder.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ozone.erasurecode.rawcoder;

import org.apache.hadoop.classification.InterfaceAudience;
import org.apache.hadoop.hdds.client.ECReplicationConfig;
import org.apache.hadoop.util.PerformanceAdvisory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * Abstract native raw encoder for all native coders to extend with.
 */
@InterfaceAudience.Private
@SuppressWarnings("checkstyle:VisibilityModifier")
abstract class AbstractNativeRawEncoder extends RawErasureEncoder {
  public static final Logger LOG =
      LoggerFactory.getLogger(AbstractNativeRawEncoder.class);

  // Protect ISA-L coder data structure in native layer from being accessed and
  // updated concurrently by the init, release and encode functions.
  protected final ReentrantReadWriteLock encoderLock =
      new ReentrantReadWriteLock();

  AbstractNativeRawEncoder(ECReplicationConfig replicationConfig) {
    super(replicationConfig);
  }

  @Override
  protected void doEncode(ByteBufferEncodingState encodingState)
      throws IOException {
    encoderLock.readLock().lock();
    try {
      int[] inputOffsets = new int[encodingState.inputs.length];
      int[] outputOffsets = new int[encodingState.outputs.length];
      int dataLen = encodingState.inputs[0].remaining();

      ByteBuffer buffer;
      for (int i = 0; i < encodingState.inputs.length; ++i) {
        buffer = encodingState.inputs[i];
        inputOffsets[i] = buffer.position();
      }

      for (int i = 0; i < encodingState.outputs.length; ++i) {
        buffer = encodingState.outputs[i];
        outputOffsets[i] = buffer.position();
      }

      performEncodeImpl(encodingState.inputs, inputOffsets, dataLen,
          encodingState.outputs, outputOffsets);
    } finally {
      encoderLock.readLock().unlock();
    }
  }

  protected abstract void performEncodeImpl(
          ByteBuffer[] inputs, int[] inputOffsets,
          int dataLen, ByteBuffer[] outputs, int[] outputOffsets)
      throws IOException;

  @Override
  protected void doEncode(ByteArrayEncodingState encodingState)
      throws IOException {
    PerformanceAdvisory.LOG.debug("convertToByteBufferState is invoked, " +
        "not efficiently. Please use direct ByteBuffer inputs/outputs");

    ByteBufferEncodingState bbeState = encodingState.convertToByteBufferState();
    doEncode(bbeState);

    for (int i = 0; i < encodingState.outputs.length; i++) {
      bbeState.outputs[i].get(encodingState.outputs[i],
          encodingState.outputOffsets[i], encodingState.encodeLength);
    }
  }

  @Override
  public boolean preferDirectBuffer() {
    return true;
  }
}

==> ozone-1.4.1-src.tar.gz: ozone-1.4.1-src/hadoop-hdds/erasurecode/src/main/java/org/apache/ozone/erasurecode/rawcoder/AbstractNativeRawEncoder.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ozone.erasurecode.rawcoder;

import org.apache.hadoop.hdds.annotation.InterfaceAudience;
import org.apache.hadoop.hdds.client.ECReplicationConfig;
import org.apache.hadoop.util.PerformanceAdvisory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * Abstract native raw encoder for all native coders to extend with.
 */
@InterfaceAudience.Private
@SuppressWarnings("checkstyle:VisibilityModifier")
abstract class AbstractNativeRawEncoder extends RawErasureEncoder {
  public static final Logger LOG =
      LoggerFactory.getLogger(AbstractNativeRawEncoder.class);

  // Protect ISA-L coder data structure in native layer from being accessed and
  // updated concurrently by the init, release and encode functions.
  protected final ReentrantReadWriteLock encoderLock =
      new ReentrantReadWriteLock();

  AbstractNativeRawEncoder(ECReplicationConfig replicationConfig) {
    super(replicationConfig);
  }

  @Override
  protected void doEncode(ByteBufferEncodingState encodingState)
      throws IOException {
    encoderLock.readLock().lock();
    try {
      int[] inputOffsets = new int[encodingState.inputs.length];
      int[] outputOffsets = new int[encodingState.outputs.length];
      int dataLen = encodingState.inputs[0].remaining();

      ByteBuffer buffer;
      for (int i = 0; i < encodingState.inputs.length; ++i) {
        buffer = encodingState.inputs[i];
        inputOffsets[i] = buffer.position();
      }

      for (int i = 0; i < encodingState.outputs.length; ++i) {
        buffer = encodingState.outputs[i];
        outputOffsets[i] = buffer.position();
      }

      performEncodeImpl(encodingState.inputs, inputOffsets, dataLen,
          encodingState.outputs, outputOffsets);
    } finally {
      encoderLock.readLock().unlock();
    }
  }

  protected abstract void performEncodeImpl(
          ByteBuffer[] inputs, int[] inputOffsets,
          int dataLen, ByteBuffer[] outputs, int[] outputOffsets)
      throws IOException;

  @Override
  protected void doEncode(ByteArrayEncodingState encodingState)
      throws IOException {
    PerformanceAdvisory.LOG.debug("convertToByteBufferState is invoked, " +
        "not efficiently. Please use direct ByteBuffer inputs/outputs");

    ByteBufferEncodingState bbeState = encodingState.convertToByteBufferState();
    doEncode(bbeState);

    for (int i = 0; i < encodingState.outputs.length; i++) {
      bbeState.outputs[i].get(encodingState.outputs[i],
          encodingState.outputOffsets[i], encodingState.encodeLength);
    }
  }

  @Override
  public boolean preferDirectBuffer() {
    return true;
  }
}
